@@ -1,5 +1,5 @@
 /*
- * "$Id: rastertoprinter.c,v 1.19.4.11 2003/12/02 01:48:23 rlk Exp $"
+ * "$Id: rastertoprinter.c,v 1.19.4.15 2004/03/26 03:29:39 rlk Exp $"
  *
  *   GIMP-print based raster filter for the Common UNIX Printing System.
  *
@@ -348,7 +348,8 @@ main(int  argc,				/* I - Number of command-line arguments */
 
     cups.row = 0;
 
-    fprintf(stderr, "PAGE: %d 1\n", cups.page);
+    fprintf(stderr, "PAGE: %d 1\n", cups.page + 1);
+    /* use 1-based page logging */
 
    /*
     * Debugging info...
@@ -675,6 +676,8 @@ Image_get_row(stp_image_t   *image,	/* I - Image */
   int		i;			/* Looping var */
   int 		bytes_per_line;
   int		margin;
+  unsigned char *orig = data;
+  static int warned = 0;
 
 
   if ((cups = (cups_image_t *)(image->rep)) == NULL)
@@ -707,6 +710,30 @@ Image_get_row(stp_image_t   *image,	/* I - Image */
       else
 	memset(data, ((1 << CHAR_BIT) - 1), bytes_per_line);
     }
+
+  /*
+   * This exists to print non-ADSC input which has messed up the job
+   * input, such as that generated by psnup.
+   */
+  data = orig;
+  if (cups->header.cupsBitsPerPixel == 1)
+    {
+      if (warned == 0)
+	{
+	  fprintf(stderr,
+		  "WARNING: GIMP-PRINT detected broken job options.  "
+		  "Output quality is degraded.  Are you using psnup or non-ADSC PostScript?\n");
+	  warned = 1;
+	}
+      for (i = cups->width - 1; i >= 0; i--)
+	{
+	  if ( (data[i/8] >> (7 - i%8)) &0x1)
+	    data[i]=255;
+	  else
+	    data[i]=0;
+	}
+    }
+
   return Image_status;
 }
 
@@ -756,7 +783,8 @@ Image_note_progress(stp_image_t *image,	/* I - Image */
     return;
 
   fprintf(stderr, "INFO: Printing page %d, %.0f%%\n",
-          cups->page, 100.0 * current / total);
+          cups->page +1, 100.0 * current / total);
+    /* cups->page + 1 because users expect 1-based counting */
 }
 
 
@@ -773,7 +801,8 @@ Image_progress_conclude(stp_image_t *image)	/* I - Image */
   if ((cups = (cups_image_t *)(image->rep)) == NULL)
     return;
 
-  fprintf(stderr, "INFO: Finished page %d...\n", cups->page);
+  fprintf(stderr, "INFO: Finished page %d...\n", cups->page + 1);
+  /* cups->page + 1 because users expect 1-based counting */
 }
 
 
@@ -790,7 +819,8 @@ Image_progress_init(stp_image_t *image)/* I - Image */
   if ((cups = (cups_image_t *)(image->rep)) == NULL)
     return;
 
-  fprintf(stderr, "INFO: Starting page %d...\n", cups->page);
+  fprintf(stderr, "INFO: Starting page %d...\n", cups->page + 1);
+  /* cups->page + 1 because users expect 1-based counting */
 }
 
 
@@ -846,5 +876,5 @@ Image_width(stp_image_t *image)	/* I - Image */
 
 
 /*
- * End of "$Id: rastertoprinter.c,v 1.19.4.11 2003/12/02 01:48:23 rlk Exp $".
+ * End of "$Id: rastertoprinter.c,v 1.19.4.15 2004/03/26 03:29:39 rlk Exp $".
  */
