@@ -1,5 +1,5 @@
 /*
- * "$Id: escputil.c,v 1.39.2.16 2003/11/01 22:21:12 rlk Exp $"
+ * "$Id: escputil.c,v 1.39.2.18 2004/05/08 19:47:36 rlk Exp $"
  *
  *   Printer maintenance utility for EPSON Stylus (R) printers
  *
@@ -266,6 +266,7 @@ stp_printer_t printer_list[] =
   { "895",	N_("Stylus Photo 895"),	3,	15,	0,	0,	0 },
   { "915",	N_("Stylus Photo 915"),	3,	15,	0,	0,	0 },
   { "925",	N_("Stylus Photo 925"),	3,	15,	0,	0,	0 },
+  { "935",	N_("Stylus Photo 935"),	3,	15,	0,	0,	0 },
   { "950",	N_("Stylus Photo 950"),	4,	15,	0,	0,	0 },
   { "960",	N_("Stylus Photo 960"),	4,	15,	0,	0,	0 },
   { "1200",	N_("Stylus Photo 1200"),3,	15,	0,	0,	0 },
@@ -570,8 +571,11 @@ read_from_printer(int fd, char *buf, int bufsize)
 	break; /* poll error */
 #endif
       status = read(fd, buf, bufsize - 1);
-      if (status < 0 && errno == EAGAIN)
-	status = 0; /* not an error (read would have blocked) */
+      if (status == 0 || (status < 0 && errno == EAGAIN))
+	{
+	  sleep(1);
+	  status = 0; /* not an error (read would have blocked) */
+	}
     }
   while ((status == 0) && (--retry != 0));
 
@@ -645,6 +649,7 @@ do_ink_level(void)
 {
   int fd;
   int status;
+  int retry = 6;
   char buf[1024];
   char *ind;
   int i;
@@ -653,30 +658,44 @@ do_ink_level(void)
       fprintf(stderr,_("Obtaining ink levels requires using a raw device.\n"));
       exit(1);
     }
-  fd = open(raw_device, O_RDWR, 0666);
-  if (fd == -1)
-    {
-      fprintf(stderr, _("Cannot open %s read/write: %s\n"), raw_device,
-	      strerror(errno));
-      exit(1);
-    }
-  initialize_print_cmd();
-  do_remote_cmd("ST", 2, 0, 1);
-  add_resets(2);
-  if (write(fd, printer_cmd, bufpos) < bufpos)
-    {
-      fprintf(stderr, _("Cannot write to %s: %s\n"), raw_device,
-	      strerror(errno));
-      exit(1);
-    }
-  status = read_from_printer(fd, buf, 1024);
-  if (status < 0)
-    exit(1);
-  ind = buf;
   do
-    ind = strchr(ind, 'I');
-  while (ind && ind[1] != 'Q' && (ind[1] != '\0' && ind[2] != ':'));
-  if (!ind || ind[1] != 'Q' || ind[2] != ':')
+    {
+      if (retry <= 4)
+	sleep(1);
+      fd = open(raw_device, O_RDWR, 0666);
+      if (fd == -1)
+	{
+	  fprintf(stderr, _("Cannot open %s read/write: %s\n"), raw_device,
+		  strerror(errno));
+	  exit(1);
+	}
+      add_resets(2);
+      initialize_print_cmd();
+      if (isnew && !(retry & 1))
+	do_remote_cmd("IQ", 1, 1);
+      else
+	do_remote_cmd("ST", 2, 0, 1);
+      add_resets(2);
+      if (write(fd, printer_cmd, bufpos) < bufpos)
+	{
+	  fprintf(stderr, _("Cannot write to %s: %s\n"), raw_device,
+		  strerror(errno));
+	  exit(1);
+	}
+      status = read_from_printer(fd, buf, 1024);
+      if (status < 0)
+	exit(1);
+      (void) close(fd);
+      ind = buf;
+      do
+	ind = strchr(ind, 'I');
+      while (ind && ind[1] != 'Q' && (ind[1] != '\0' && ind[2] != ':'));
+      if (!ind || ind[1] != 'Q' || ind[2] != ':' || ind[3] == ';')
+	{
+	  ind = NULL;
+	}
+    } while (--retry != 0 && !ind);
+  if (!ind)
     {
       fprintf(stderr, _("Cannot parse output from printer\n"));
       exit(1);
@@ -703,12 +722,6 @@ do_ink_level(void)
       printf("%20s    %3d\n", _(colors[i]), val);
       ind += 2;
     }
-  initialize_print_cmd();
-  do_remote_cmd("ST", 2, 0, 0);
-  add_resets(2);
-  (void) write(fd, printer_cmd, bufpos);
-  (void) read(fd, buf, 1024);
-  (void) close(fd);
   exit(0);
 }
 
