@@ -1,5 +1,5 @@
 /*
- * "$Id: print-util.c,v 1.53.2.18 2003/01/24 02:52:18 rlk Exp $"
+ * "$Id: print-util.c,v 1.53.2.21 2004/02/16 21:08:33 rlk Exp $"
  *
  *   Print plug-in driver utility functions for the GIMP.
  *
@@ -560,6 +560,8 @@ static stp_internal_papersize_t paper_sizes[] =
     864, 1296, 0, 0, 0, 0, PAPERSIZE_ENGLISH },
   { "SuperB",		N_ ("Super B 13x19"),
     936, 1368, 0, 0, 0, 0, PAPERSIZE_ENGLISH },
+  { "EngineeringC",	N_ ("Engineering C 17x22"),
+    1224, 1584, 0, 0, 0, 0, PAPERSIZE_ENGLISH },
 
   /* Other common photographic paper sizes */
   { "w576h864",		N_ ("8x12"),
@@ -597,6 +599,8 @@ static stp_internal_papersize_t paper_sizes[] =
     4768, 6749, 0, 0, 0, 0, PAPERSIZE_METRIC },	/* 1682mm x 2378mm */
   { "w3370h4768",	N_ ("2A"),
     3370, 4768, 0, 0, 0, 0, PAPERSIZE_METRIC },	/* 1189mm x 1682mm */
+  { "SuperA0",		N_ ("Super A0"),
+    2590, 3662, 0, 0, 0, 0, PAPERSIZE_METRIC },	/*  914mm x 1292mm */
   { "A0",		N_ ("A0"),
     2384, 3370, 0, 0, 0, 0, PAPERSIZE_METRIC },	/*  841mm x 1189mm */
   { "A1",		N_ ("A1"),
@@ -679,6 +683,8 @@ static stp_internal_papersize_t paper_sizes[] =
   { "ISOB10",		N_ ("B10 ISO"),
     87,  124, 0, 0, 0, 0, PAPERSIZE_METRIC },	/*   31mm x   44mm */
 
+  { "SuperB0",		N_ ("Super B0 JIS"),
+    3167, 4478, 0, 0, 0, 0, PAPERSIZE_METRIC },
   { "B0",		N_ ("B0 JIS"),
     2919, 4127, 0, 0, 0, 0, PAPERSIZE_METRIC },
   { "B1",		N_ ("B1 JIS"),
@@ -801,6 +807,8 @@ static stp_internal_papersize_t paper_sizes[] =
     365, 561, 0, 0, 0, 0, PAPERSIZE_METRIC }, /* 129mm x 198mm */
 
   /* Miscellaneous sizes */
+  { "w283h425",		N_ ("10cm x 15cm"),
+    283, 425, 0, 0, 0, 0, PAPERSIZE_METRIC }, /* 100 x 150 mm */
   { "w283h420",		N_ ("Hagaki Card"),
     283, 420, 0, 0, 0, 0, PAPERSIZE_METRIC }, /* 100 x 148 mm */
   { "w420h567",		N_ ("Oufuku Card"),
