@@ -1,4 +1,4 @@
-/* $Id: unprint.c,v 1.22.4.1 2002/07/23 00:54:08 rlk Exp $ */
+/* $Id: unprint.c,v 1.22.4.2 2003/12/02 01:51:01 rlk Exp $ */
 /*
  * Generate PPM files from printer output
  *
@@ -39,7 +39,7 @@
  */
 typedef struct {
   unsigned char unidirectional;
-  unsigned char microweave;
+  unsigned char interleave;
   int page_management_units; /* dpi */
   int relative_horizontal_units; /* dpi */
   int absolute_horizontal_units; /* dpi, assumed to be >= relative */
@@ -555,7 +555,7 @@ update_page(unsigned char *buf, /* I - pixel data               */
       page = (line_type **) xcalloc(pstate.bottom_margin - pstate.top_margin,
 				    sizeof(line_type *));
     }
-  if (pstate.microweave)
+  if (pstate.interleave)
     sep = 1;
   else
     sep = pstate.nozzle_separation;
@@ -827,7 +827,7 @@ parse_escp2_extended(FILE *fp_r)
       break;
     case 'G': /* select graphics mode */
       /* FIXME: this is supposed to have more side effects */
-      pstate.microweave = 0;
+      pstate.interleave = 0;
       pstate.dotsize = 0;
       pstate.bpp = 1;
       break;
@@ -864,11 +864,11 @@ parse_escp2_extended(FILE *fp_r)
 	  break;
 	}
       break;
-    case 'i': /* set MicroWeave mode */
+    case 'i': /* set Interleave mode */
       if (bufsize != 1)
-	fprintf(stderr,"Malformed microweave setting command.\n");
+	fprintf(stderr,"Malformed interleave setting command.\n");
       else
-	pstate.microweave = buf[0] % 0x30;
+	pstate.interleave = buf[0] % 0x30;
       break;
     case 'e': /* set dot size */
       if ((bufsize != 2) || (buf[0] != 0))
@@ -1101,7 +1101,7 @@ parse_escp2_command(FILE *fp_r)
       else
 	{
 	  pstate.unidirectional = 0;
-	  pstate.microweave = 0;
+	  pstate.interleave = 0;
 	  pstate.dotsize = 0;
 	  pstate.bpp = 1;
 	  pstate.page_management_units = 360;
@@ -1346,7 +1346,7 @@ parse_canon(FILE *fp_r)
 	 continue;
        } else {
 	 pstate.unidirectional=0;
-	 pstate.microweave=0;
+	 pstate.interleave=0;
 	 pstate.dotsize=0;
 	 pstate.bpp=1;
 	 pstate.page_management_units=360;
